/**
 * Created by John on 11/17/2016.
 *
 * Common routines for handling RabbitMQ queues
 */

function RabbitHelper(winston, config) {
    var self = this;
    this.rabbit = require("rabbot");
    this.winston = winston;
    this.logger = (winston) ? winston.loggers.get('main') : console;
    // it can make a lot of sense to share topology definition across
    // services that will be using the same topology to avoid
    // scenarios where you have race conditions around when
    // exchanges, queues or bindings are in place
    require("../config/RabbitTopology.js")(this.rabbit, null, config)
        .then(function() {
            self.logger.debug("Rabbit Topology Loaded");
        });

    this.rabbit.on("unreachable", function() {
        self.logger.error("RABBIT INIT :(");
        process.exit();
    });
}

/**
 * NOTE: Only used for MOCHA TESTS!!!!!! (for now)
 *
 * @param key
 * @param data
 * @param cb
 */
RabbitHelper.prototype.publishGeneric = function(key, data, cb) {
    var i;
    var self = this;
    var send = function(x, idx, max) {
        var direction = key;
        self.rabbit.publish("topic-fresenius-x", {
            routingKey: direction,
            type: direction,
            body: x,
            persistent: true
        }).then(function() {
            console.log("published " + key + " message", x);
            var out = {
                msg: x,
                idx: idx,
                total: max
            };
            if (cb) cb(null, out);
        }).catch(function(err){
            if (cb) cb(err, null);
            else throw err;
        });
    };

    if (Array.isArray(data)) {
        for (i = 0; i < data.length; i++) {
            send(data[i], i, data.length);
        }
    } else {
        send(data, 0, 1);
    }

};

RabbitHelper.prototype.publishAsn = function(msg, cb) {
    var self = this;
    var direction = "asn";
    self.rabbit.publish("topic-fresenius-x", {
        routingKey: direction,
        type: direction,
        body: msg
    }).then(function() {
        self.logger.debug("published ASN message");
        if (cb) cb(null, msg);
    }).catch(function(err){
        if (cb) cb(err, null);
        else throw err;
    });
};

RabbitHelper.prototype.publishPO = function(msg, cb) {
    var self = this;
    var direction = "po";
    self.rabbit.publish("topic-fresenius-x", {
        routingKey: direction,
        type: direction,
        body: msg
    }).then(function() {
        self.logger.debug("published PO message");
        if (cb) cb(null, msg);
    }).catch(function(err){
        if (cb) cb(err, null);
        else throw err;
    });
};

RabbitHelper.prototype.cleanUp = function(cb) {
    this.rabbit.shutdown().then(function(){
        if (cb) cb(null, null);
    }).catch(function(err){
        if (cb) cb(err, null);
        else throw err;
    });
};

module.exports = RabbitHelper;