/**
 * Created by John on 11/17/2016.
 *
 * Common routines for handling Amazon SQS queues
 */
var hash = require('object-hash');
var Producer = require("sqs-producer");
var AWS = require("aws-sdk");

function SqsHelper(winston, config) {
    this.Squiss = require("squiss");
    this.winston = winston;
    this.logger = (winston) ? winston.loggers.get('main') : console;
    if (!console.debug) {
        console.debug = console.log;
    }
    // it can make a lot of sense to share topology definition across
    // services that will be using the same topology to avoid
    // scenarios where you have race conditions around when
    // exchanges, queues or bindings are in place
    this.sqs = require("../config/SquissTopology")(this.Squiss, config);
    this.publisher = Producer.create(config.sqsConfig);
    this.publisher.sqs.config.credentials = new AWS.Credentials(config.sqsConfig.accessKeyId, config.sqsConfig.secretAccessKey);
}


/**
 * NOTE: Only used for MOCHA TESTS!!!!!! (for now)
 *
 * @param key
 * @param data
 * @param cb
 */
SqsHelper.prototype.publishGeneric = function(key, data, cb) {
    var i;
    var self = this;
    var send = function(x, idx, max) {
        var msg = {
            id: self.getId(key, x),
            body: JSON.stringify(x),
            groupId: key,
            deduplicationId: hash.MD5(x) // typically a hash of the message body
        };
        self.publisher.send(msg, function(err, sqsResponse) {
            if (err) {
                self.logger.error(err);
                if (cb) cb(err, null);
            } else {
                self.logger.debug("published " + key + " message", msg.body);
                var out = {
                    msg: x,
                    idx: idx,
                    total: max,
                    res: sqsResponse
                };
                if (cb) cb(null, out);
            }
        });
    };
    var sendBulk = function(x) {
        self.sqs[key].sendMessages(x).then(function(sqsResponse) {
            self.logger.debug("published " + sqsResponse.Successful + " messages", sqsResponse.Failed);
            if (cb) cb(null, sqsResponse);
        });
    };

    if (Array.isArray(data)) {
        // sendBulk(data)  CANNOT USE, DeDupe must be unique per message
        for (i = 0; i < data.length; i++) {
            send(data[i], i, data.length);
        }
    } else {
        send(data);
    }
};

SqsHelper.prototype.publishAsn = function(msg, cb) {
    var self = this;
    self.publishGeneric('asn', msg, cb);
};

SqsHelper.prototype.publishPO = function(msg, cb) {
    var self = this;
    self.publishGeneric('po', msg, cb);
};


SqsHelper.prototype.pollASN = function(cb) {
    this.sqs.asn.on('message', function(msg) {
        cb(null, msg);
    });

    this.sqs.asn.on('error', function(err) {
        cb(err, null);
    });

    this.sqs.asn.on('delError', function(err) {
        cb(err, null);
    });

    this.sqs.asn.start();
};

SqsHelper.prototype.pollPO = function(cb) {
    this.sqs.po.on('message', function(msg) {
        cb(null, msg);
    });

    this.sqs.po.on('error', function(err) {
        cb(err, null);
    });

    this.sqs.po.start();
};

SqsHelper.prototype.cleanUp = function(cb) {
    this.sqs.asn.stop(true);
    //this.sqs.po.stop(true);
};

SqsHelper.prototype.getId = function (key, obj) {
    var id  = "";
    switch (key) {
        case "asn": {
            id = obj.asnId + obj.lineNumber;
            break;
        }
        default: {
            id = obj.id;
        }
    }
    return id;
};

module.exports = SqsHelper;